/* Project name:
     JPEG direct load
 * Copyright:
     (c) -
 * Created by:
      Stevan A. Milinkovic
      Union University, School of Computing
      Knez Mihailova 6, 11000 Belgrade, Serbia
      e-mail: smilinkovic@raf.edu.rs
 * Revision History:
       - 2012-07-30 v 0.0.0.1
 */
 
typedef int                 INT;
typedef unsigned int        UINT;

typedef char                CHAR;
typedef unsigned char       UCHAR;
typedef unsigned char       BYTE;

typedef int                 SHORT;
typedef unsigned int        USHORT;
typedef unsigned int        WORD;
typedef unsigned int        WCHAR;

typedef long                LONG;
typedef unsigned long       ULONG;
typedef unsigned long       DWORD;


// Error codes
typedef enum {
        OK = 0,             // 0: Succeeded
        ERR_INTR,           // 1: Interrupted by output function
        ERR_INP,            // 2: Device error or wrong termination of input
        ERR_MEM1,           // 3: Insufficient memory pool for the image
        ERR_MEM2,           // 4: Insufficient stream input buffer
        ERR_PAR,            // 5: Parameter error
        ERR_FMT1,           // 6: Data format error
        ERR_FMT2,           // 7: Right format but not supported
        ERR_FMT3            // 8: Not supported JPEG standard
} RESULT;


typedef struct {
        WORD left, right, top, bottom;
} RECT;


// JPEG structure 
typedef struct JPGS JPGS;

static  struct JPGS {
        UINT dctr;                                // Number of bytes available in input buffer
        BYTE* dptr;                               // Current data read pointer
        BYTE* inbuf;                              // Input buffer
        BYTE dmsk;                                // Current bit in the current byte
        BYTE scale;                               // Output scaling ratio
        BYTE msx, msy;                            // MCU size in unit of block (width, height)
        BYTE qtid[3];                             // Quantization table ID of each component
        SHORT dcv[3];                             // Previous DC element of each component
        WORD nrst;                                // Restart inverval
        UINT width, height;                       // Size of the input image (pixels)
        BYTE* huffbits[2][2];                     // Huffman bit distribution tables [id][dcac]
        WORD* huffcode[2][2];                     // Huffman code word tables [id][dcac]
        BYTE* huffdata[2][2];                     // Huffman decoded data tables [id][dcac]
        LONG* qttbl[4];                           // Dequantizer tables [id]
        void* workbuf;                            // Working buffer for IDCT and RGB
        BYTE* mcubuf;                             // Working buffer for MCU
        void* pool;                               // Pointer to available memory pool
        UINT p_size;                              // Size of momory pool (bytes available)
        UINT (*virtual_in)(JPGS*, BYTE*, UINT);   // Pointer to JPEG input function
        UINT (*virtual_out)(JPGS*, void*, RECT*); // Pointer to RGB output function
        void* device;                             // Pointer to I/O device identifiler
};

#define  LDB_WORD(ptr)  (WORD)(((WORD)*((BYTE*)(ptr))<<8)|(WORD)*(BYTE*)((ptr)+1))
#define  STREAM_BUF      512                      // Size of input buffer